import * as i0 from '@angular/core';
import { signal, computed, Component, Input, Pipe, TemplateRef, ContentChild, EventEmitter, effect, ViewChild, HostBinding, Injector, Injectable } from '@angular/core';
import { toSignal, toObservable } from '@angular/core/rxjs-interop';
import { map, fromEvent, from, filter, switchMap, combineLatest, firstValueFrom } from 'rxjs';
import { style, animate, transition, trigger } from '@angular/animations';
import * as i2 from '@angular/cdk/overlay';
import { FlexibleConnectedPositionStrategy, Overlay } from '@angular/cdk/overlay';
import { TemplatePortal, ComponentPortal } from '@angular/cdk/portal';
import { NgTemplateOutlet, NgClass, JsonPipe } from '@angular/common';
import { takeUntil } from 'rxjs/operators';

class ArrowComponent {
    get direction() {
        return this._direction();
    }
    set direction(value) {
        this._direction.set(value);
    }
    get pointToElement() {
        return this._pointToElement();
    }
    set pointToElement(value) {
        this._pointToElement.set(value);
    }
    constructor(host, zone) {
        this.host = host;
        this._direction = signal(undefined);
        this._pointToElement = signal(null);
        this.viewBox = computed(() => {
            const bounds = this.bounds();
            if (!bounds) {
                return `0 0 0 0`;
            }
            return `0 0 ${bounds.width} ${bounds.height}`;
        });
        this.pathData = computed(() => {
            const bounds = this.bounds() ?? { x: 0, y: 0, width: 0, height: 0 };
            const absCenterX = bounds.x + bounds.width / 2;
            const absCenterY = bounds.y + bounds.height / 2;
            const padding = 5;
            const minX = padding;
            const minY = padding;
            const maxX = bounds.width - padding;
            const maxY = bounds.height - padding;
            const width = maxX - minX;
            const height = maxY - minY;
            const halfWidth = width / 2;
            const halfHeight = height / 2;
            const direction = this._direction();
            const pointToElement = this._pointToElement();
            const elementCenterPoint = pointToElement?.getBoundingClientRect() ?? {
                x: 0,
                y: 0,
                width: 0,
                height: 0,
            };
            const elementCenterX = elementCenterPoint.x + elementCenterPoint.width / 2;
            const elementCenterY = elementCenterPoint.y + elementCenterPoint.height / 2;
            const dx = elementCenterX - absCenterX;
            const dy = elementCenterY - absCenterY;
            let fromX = 0, fromY = 0, toX = 0, toY = 0, controlPointX = 0, controlPointY = 0;
            switch (direction) {
                case 'top':
                    fromX = halfWidth;
                    fromY = minY;
                    toX = halfWidth + dx;
                    toY = maxY;
                    controlPointX = halfWidth;
                    controlPointY = halfHeight;
                    break;
                case 'bottom':
                    fromX = halfWidth;
                    fromY = maxY;
                    toX = halfWidth + dx;
                    toY = minY;
                    controlPointX = halfWidth;
                    controlPointY = halfHeight;
                    break;
                case 'left':
                    fromX = halfWidth;
                    fromY = minY;
                    toX = maxX;
                    toY = halfHeight + dy;
                    controlPointX = halfWidth;
                    controlPointY = halfHeight;
                    break;
                case 'right':
                    fromX = halfWidth;
                    fromY = minY;
                    toX = minX;
                    toY = halfHeight + dy;
                    controlPointX = halfWidth;
                    controlPointY = halfHeight;
                    break;
            }
            toX = Math.max(minX, Math.min(maxX, toX));
            toY = Math.max(minY, Math.min(maxY, toY));
            return `M${fromX},${fromY} Q${controlPointX},${controlPointY} ${toX},${toY}`;
        });
        this.bounds = signal(undefined);
        const observer = new ResizeObserver(() => {
            zone.run(() => {
                this.bounds.set(host.nativeElement.getBoundingClientRect());
            });
        });
        observer.observe(host.nativeElement);
        this.resizeObserver = observer;
    }
    ngOnDestroy() {
        this.resizeObserver.unobserve(this.host.nativeElement);
        this.resizeObserver.disconnect();
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "17.0.8", ngImport: i0, type: ArrowComponent, deps: [{ token: i0.ElementRef }, { token: i0.NgZone }], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "14.0.0", version: "17.0.8", type: ArrowComponent, isStandalone: true, selector: "lib-arrow", inputs: { direction: "direction", pointToElement: "pointToElement" }, ngImport: i0, template: `
    <svg [attr.viewBox]="viewBox()">
      <defs>
        <marker
          id="arrowMarker"
          viewBox="0 0 36 21"
          refX="21"
          refY="10"
          markerUnits="strokeWidth"
          orient="auto"
          markerWidth="16"
          markerHeight="12"
        >
          <path id="polyline" d="M0,0 c30,11 30,9 0,20" />
        </marker>
      </defs>
      <path [attr.d]="pathData()" marker-end="url(#arrowMarker)" />
    </svg>
  `, isInline: true, styles: [":host{display:block;width:100%;height:100px}path{fill:none;stroke:#fff;stroke-width:2}\n"] }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "17.0.8", ngImport: i0, type: ArrowComponent, decorators: [{
            type: Component,
            args: [{ selector: 'lib-arrow', standalone: true, template: `
    <svg [attr.viewBox]="viewBox()">
      <defs>
        <marker
          id="arrowMarker"
          viewBox="0 0 36 21"
          refX="21"
          refY="10"
          markerUnits="strokeWidth"
          orient="auto"
          markerWidth="16"
          markerHeight="12"
        >
          <path id="polyline" d="M0,0 c30,11 30,9 0,20" />
        </marker>
      </defs>
      <path [attr.d]="pathData()" marker-end="url(#arrowMarker)" />
    </svg>
  `, styles: [":host{display:block;width:100%;height:100px}path{fill:none;stroke:#fff;stroke-width:2}\n"] }]
        }], ctorParameters: () => [{ type: i0.ElementRef }, { type: i0.NgZone }], propDecorators: { direction: [{
                type: Input
            }], pointToElement: [{
                type: Input
            }] } });

class ForceFieldComponent {
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "17.0.8", ngImport: i0, type: ForceFieldComponent, deps: [], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "14.0.0", version: "17.0.8", type: ForceFieldComponent, isStandalone: true, selector: "lib-force-field", inputs: { size: "size", position: "position" }, host: { properties: { "style.width": "size.width + \"px\"", "style.height": "size.height + \"px\"", "style.minWidth": "size.width + \"px\"", "style.minHeight": "size.height + \"px\"", "style.maxWidth": "size.width + \"px\"", "style.maxHeight": "size.height + \"px\"", "style.top": "position.y + \"px\"", "style.left": "position.x + \"px\"" } }, ngImport: i0, template: ``, isInline: true, styles: [":host{display:block;position:fixed;transition:top .5s ease-in-out,left .5s ease-in-out}\n"] }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "17.0.8", ngImport: i0, type: ForceFieldComponent, decorators: [{
            type: Component,
            args: [{ selector: 'lib-force-field', standalone: true, template: ``, host: {
                        '[style.width]': 'size.width + "px"',
                        '[style.height]': 'size.height + "px"',
                        '[style.minWidth]': 'size.width + "px"',
                        '[style.minHeight]': 'size.height + "px"',
                        '[style.maxWidth]': 'size.width + "px"',
                        '[style.maxHeight]': 'size.height + "px"',
                        '[style.top]': 'position.y + "px"',
                        '[style.left]': 'position.x + "px"',
                    }, styles: [":host{display:block;position:fixed;transition:top .5s ease-in-out,left .5s ease-in-out}\n"] }]
        }], propDecorators: { size: [{
                type: Input
            }], position: [{
                type: Input
            }] } });

class Elements {
    get appRoot() {
        return document.querySelector('[ng-version]');
    }
    get body() {
        return document.body;
    }
}
function provideWindow() {
    return { provide: Window, useValue: window };
}

function getDirection(position) {
    if (!position) {
        return undefined;
    }
    if (position.originX === 'start' && position.originY === 'center') {
        return 'left';
    }
    if (position.originX === 'end' && position.originY === 'center') {
        return 'right';
    }
    if (position.originX === 'center' && position.originY === 'top') {
        return 'top';
    }
    if (position.originX === 'center' && position.originY === 'bottom') {
        return 'bottom';
    }
    console.warn('Unknown position', position);
    return undefined;
}
class DirectionPipe {
    transform(position) {
        return getDirection(position);
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "17.0.8", ngImport: i0, type: DirectionPipe, deps: [], target: i0.ɵɵFactoryTarget.Pipe }); }
    static { this.ɵpipe = i0.ɵɵngDeclarePipe({ minVersion: "14.0.0", version: "17.0.8", ngImport: i0, type: DirectionPipe, isStandalone: true, name: "direction" }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "17.0.8", ngImport: i0, type: DirectionPipe, decorators: [{
            type: Pipe,
            args: [{
                    name: 'direction',
                    standalone: true,
                    pure: true,
                }]
        }] });

const rightCenter = {
    // right center
    originX: 'end',
    overlayX: 'start',
    originY: 'center',
    overlayY: 'center',
};
const rightTop = {
    // right top
    originX: 'end',
    overlayX: 'start',
    originY: 'top',
    overlayY: 'bottom',
};
const rightBottom = {
    // right bottom
    originX: 'end',
    overlayX: 'start',
    originY: 'bottom',
    overlayY: 'top',
};
const leftCenter = {
    // left center
    originX: 'start',
    overlayX: 'end',
    originY: 'center',
    overlayY: 'center',
};
const leftTop = {
    // left top
    originX: 'start',
    overlayX: 'end',
    originY: 'top',
    overlayY: 'bottom',
};
const leftBottom = {
    // left bottom
    originX: 'start',
    overlayX: 'end',
    originY: 'bottom',
    overlayY: 'top',
};
const topCenter = {
    // top center
    originX: 'center',
    overlayX: 'center',
    originY: 'top',
    overlayY: 'bottom',
};
const bottomCenter = {
    // bottom center
    originX: 'center',
    overlayX: 'center',
    originY: 'bottom',
    overlayY: 'top',
};
const ALL_POSITIONS = [
    rightCenter,
    rightTop,
    rightBottom,
    leftCenter,
    leftTop,
    leftBottom,
    topCenter,
    bottomCenter,
];
const ALL_SIDES = [rightCenter, leftCenter, topCenter, bottomCenter];

class TextOrTemplateComponent {
    constructor() {
        this._value = signal(undefined);
        this.stringValue = computed(() => {
            const value = this._value();
            return typeof value === 'string' ? value : undefined;
        });
        this.templateValue = computed(() => {
            const value = this._value();
            return value instanceof TemplateRef ? value : undefined;
        });
    }
    get value() {
        return this._value();
    }
    set value(value) {
        this._value.set(value);
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "17.0.8", ngImport: i0, type: TextOrTemplateComponent, deps: [], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.0.0", version: "17.0.8", type: TextOrTemplateComponent, isStandalone: true, selector: "lib-text-or-template", inputs: { value: "value" }, queries: [{ propertyName: "defaultTextTemplate", first: true, predicate: TemplateRef, descendants: true }], ngImport: i0, template: `
        @if (templateValue(); as template) {
            <ng-container *ngTemplateOutlet="template"></ng-container>
        } @else if (stringValue() && defaultTextTemplate) {
            <ng-container *ngTemplateOutlet="defaultTextTemplate; context: {$implicit: stringValue()}"></ng-container>
        } @else if (stringValue()) {
            <span>{{stringValue()}}</span>
        }
    `, isInline: true, styles: [""], dependencies: [{ kind: "directive", type: NgTemplateOutlet, selector: "[ngTemplateOutlet]", inputs: ["ngTemplateOutletContext", "ngTemplateOutlet", "ngTemplateOutletInjector"] }] }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "17.0.8", ngImport: i0, type: TextOrTemplateComponent, decorators: [{
            type: Component,
            args: [{ selector: 'lib-text-or-template', standalone: true, imports: [NgTemplateOutlet], template: `
        @if (templateValue(); as template) {
            <ng-container *ngTemplateOutlet="template"></ng-container>
        } @else if (stringValue() && defaultTextTemplate) {
            <ng-container *ngTemplateOutlet="defaultTextTemplate; context: {$implicit: stringValue()}"></ng-container>
        } @else if (stringValue()) {
            <span>{{stringValue()}}</span>
        }
    ` }]
        }], propDecorators: { value: [{
                type: Input
            }], defaultTextTemplate: [{
                type: ContentChild,
                args: [TemplateRef]
            }] } });

class EnjoyHintRef {
    constructor(tutorial, options = {}) {
        this.tutorial = tutorial;
        this.options = options;
        this.onClose = new EventEmitter();
    }
    close(completed = false) {
        this.onClose.emit(completed);
    }
}

class EnjoyHintComponent {
    static { this.defaultOptions = {
        padding: 5,
        fontFamily: 'sans-serif',
        backdropColor: 'black',
        overlayZIndex: undefined,
        backdropOpacity: 0.75,
        foregroundColor: 'white',
        nextButton: { text: 'Next' },
        skipButton: { text: 'Skip' },
        previousButton: { text: 'Previous' },
    }; }
    constructor(ref, window, overlayPositionBuilder, overlay, viewContainerRef, destroyedRef, zone) {
        this.ref = ref;
        this.overlayPositionBuilder = overlayPositionBuilder;
        this.overlay = overlay;
        this.viewContainerRef = viewContainerRef;
        this.destroyedRef = destroyedRef;
        this.zone = zone;
        this.instructionsWidth = computed(() => this.viewSize().width / 4);
        this.animating = signal(false);
        this.focusElement = computed(() => {
            const step = this.ref.tutorial.step();
            if (!step?.selector) {
                return null;
            }
            return document.querySelector(step.selector);
        });
        this.elementBounds = computed(() => {
            const element = this.focusElement();
            if (!element) {
                return {
                    x: 0,
                    y: 0,
                    width: 0,
                    height: 0,
                };
            }
            return element.getBoundingClientRect();
        });
        this.ffPositions = computed(() => {
            const bounds = this.elementBounds();
            const size = this.viewSize();
            return {
                left: {
                    x: -1 * size.width + bounds.x - this.options.padding,
                    y: 0,
                },
                right: {
                    x: bounds.x + bounds.width + this.options.padding,
                    y: 0,
                },
                top: {
                    x: 0,
                    y: -1 * size.height + bounds.y - this.options.padding,
                },
                bottom: {
                    x: 0,
                    y: bounds.y + bounds.height + this.options.padding,
                },
            };
        });
        this.step = computed(() => this.ref.tutorial.step());
        this.overlayRef = null;
        this.positionStrategy = computed(() => {
            const element = this.focusElement();
            if (!element) {
                return this.overlayPositionBuilder
                    .global()
                    .centerHorizontally()
                    .centerVertically();
            }
            return this.overlayPositionBuilder
                .flexibleConnectedTo(element)
                .withFlexibleDimensions(false)
                .withPositions(ALL_SIDES);
        });
        this.position = signal(undefined);
        this.options = {
            ...EnjoyHintComponent.defaultOptions,
            ...ref.options,
        };
        this.opacity = this.options.backdropOpacity.toString();
        const getSize = () => ({
            width: window.innerWidth,
            height: window.innerHeight,
        });
        this.viewSize = toSignal(fromEvent(window, 'resize').pipe(map(getSize)), {
            initialValue: getSize(),
        });
        this.thisIsDestroyed = from(new Promise((resolve) => this.destroyedRef.onDestroy(() => resolve())));
        combineLatest([toObservable(this.focusElement), toObservable(this.step)])
            .pipe(takeUntil(this.thisIsDestroyed), filter(([e, s]) => !!e && !!s), switchMap(([element, step]) => fromEvent(element, step.event)))
            .subscribe(() => {
            this.ref.tutorial.nextStep();
            if (!this.step()) {
                this.close(true);
            }
        });
        effect(() => {
            this.step(); // even though we don't use it, this is how we make the effect dependent on the step
            const overlayRef = this.overlayRef;
            overlayRef?.detach();
            overlayRef?.dispose();
            this.animating.set(true);
            setTimeout(() => this.createOverlay(), 750);
        }, { allowSignalWrites: true });
    }
    createOverlay() {
        const step = this.step();
        const positionStrategy = this.positionStrategy();
        const element = this.focusElement();
        if (!step || !positionStrategy) {
            return;
        }
        if (positionStrategy instanceof FlexibleConnectedPositionStrategy) {
            positionStrategy.positionChanges
                .pipe(takeUntil(this.thisIsDestroyed))
                .subscribe((x) => {
                this.zone.run(() => this.position.set(x.connectionPair));
            });
        }
        const overlayRef = this.overlay.create({
            positionStrategy,
            hasBackdrop: false,
        });
        const zIndex = this.options.overlayZIndex;
        if (zIndex !== undefined) {
            const globalOverlayWrapper = overlayRef.hostElement;
            globalOverlayWrapper.style.setProperty('z-index', zIndex.toString());
        }
        const overlayComponent = new TemplatePortal(this.instructions, this.viewContainerRef);
        overlayRef.attach(overlayComponent);
        this.overlayRef = overlayRef;
        this.animating.set(false);
    }
    previous() {
        this.ref.tutorial.previousStep();
    }
    next() {
        this.ref.tutorial.nextStep();
        if (!this.step()) {
            this.close(true);
        }
    }
    skip() {
        this.close(false);
    }
    close(result = false) {
        this.ref.close(result);
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "17.0.8", ngImport: i0, type: EnjoyHintComponent, deps: [{ token: EnjoyHintRef }, { token: Window }, { token: i2.OverlayPositionBuilder }, { token: i2.Overlay }, { token: i0.ViewContainerRef }, { token: i0.DestroyRef }, { token: i0.NgZone }], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.0.0", version: "17.0.8", type: EnjoyHintComponent, isStandalone: true, selector: "lib-enjoyhint", host: { properties: { "style.opacity": "this.opacity" } }, providers: [provideWindow()], viewQueries: [{ propertyName: "instructions", first: true, predicate: ["instructions"], descendants: true }], ngImport: i0, template: "@if (ffPositions(); as positions) {\n<lib-force-field\n  [size]=\"viewSize()\"\n  [position]=\"positions.top\"\n  [style.background-color]=\"options.backdropColor\"\n></lib-force-field>\n<lib-force-field\n  [size]=\"viewSize()\"\n  [position]=\"positions.left\"\n  [style.background-color]=\"options.backdropColor\"\n></lib-force-field>\n<lib-force-field\n  [size]=\"viewSize()\"\n  [position]=\"positions.right\"\n  [style.background-color]=\"options.backdropColor\"\n></lib-force-field>\n<lib-force-field\n  [size]=\"viewSize()\"\n  [position]=\"positions.bottom\"\n  [style.background-color]=\"options.backdropColor\"\n></lib-force-field>\n} @if (step()?.event === 'next') {\n<!-- If we just want them to click on the 'Next' button we want to disable events over the focused element -->\n<lib-force-field\n  [size]=\"elementBounds()\"\n  [position]=\"elementBounds()\"\n  [style.background-color]=\"'transparent'\"\n>\n</lib-force-field>\n}\n\n<ng-template #instructions>\n  <div\n    class=\"instructions\"\n    @fadeIn\n    [ngClass]=\"{\n    top: (position() | direction) === 'top',\n    left: (position() | direction) === 'left',\n    right: (position() | direction) === 'right',\n    bottom: (position() | direction) === 'bottom',\n  }\"\n  [style.maxWidth]=\"instructionsWidth()\"\n  >\n    <lib-text-or-template [value]=\"step()?.description\">\n      <ng-template let-description>\n        <label [style.fontFamily]=\"options.fontFamily\">\n          {{ description }}\n        </label>\n      </ng-template>\n    </lib-text-or-template>\n    @if (step()?.details) {\n    <lib-text-or-template [value]=\"step()?.details\">\n      <ng-template let-details>\n        <p [style.fontFamily]=\"options.fontFamily\">\n          {{ details }}\n        </p>\n      </ng-template>\n    </lib-text-or-template>\n    }\n    @if (focusElement(); as element) {\n      <lib-arrow [direction]=\"position() | direction\" [pointToElement]=\"focusElement()\"></lib-arrow>\n    }\n    <div class=\"buttons\">\n      @if (ref.tutorial.hasPrevious()) {\n      <button\n        class=\"previous\"\n        (click)=\"previous()\"\n        [style.fontFamily]=\"options.fontFamily\"\n        [ngClass]=\"step()?.previousButton?.className\"\n      >\n        {{ step()?.previousButton?.text ?? options.previousButton.text }}\n      </button>\n      } @if (step()?.event === 'next') {\n      <button\n        class=\"next\"\n        (click)=\"next()\"\n        [style.fontFamily]=\"options.fontFamily\"\n        [ngClass]=\"step()?.nextButton?.className\"\n      >\n        {{ step()?.nextButton?.text ?? options.nextButton.text }}\n      </button>\n      } @if (!step()?.hideSkip) {\n      <button\n        class=\"skip\"\n        (click)=\"skip()\"\n        [style.fontFamily]=\"options.fontFamily\"\n        [ngClass]=\"step()?.skipButton?.className\"\n      >\n        {{ step()?.skipButton?.text ?? options.skipButton.text }}\n      </button>\n      }\n    </div>\n  </div>\n</ng-template>\n", styles: ["label{color:#fff;font-size:2rem}p{color:#fff;font-size:1.5rem}.instructions{display:grid;grid-template-rows:repeat(4,auto);grid-template-columns:auto;gap:.5rem;justify-content:start;justify-items:start;padding:.75rem;max-width:25vw}.instructions label{grid-column:1;grid-row:1}.instructions p{grid-column:1;grid-row:2}.instructions lib-arrow{grid-column:1;grid-row:3}.instructions .buttons{grid-column:1;grid-row:4}.instructions.left{justify-content:end;justify-items:end}.instructions.right{justify-content:start;justify-items:start}.instructions.top{justify-content:center;justify-items:center}.instructions.top .buttons{grid-row:1}.instructions.top label{grid-row:2}.instructions.top p{grid-row:3}.instructions.top lib-arrow{grid-row:4}.instructions.bottom{justify-content:center;justify-items:center}.instructions.bottom lib-arrow{grid-row:1}.instructions.bottom label{grid-row:2}.instructions.bottom p{grid-row:3}.instructions.bottom .buttons{grid-row:4}.buttons{display:flex;flex-direction:row;gap:.5rem}.buttons button{background-color:transparent;border:1px solid white;border-radius:5px;color:#fff;font-size:.75rem;width:5rem;padding:.5rem;transition:all .25s ease-in-out;cursor:pointer}.buttons button.next{background-color:green;color:#fff}.buttons button:hover{background-color:#fff;color:#000}\n"], dependencies: [{ kind: "component", type: ForceFieldComponent, selector: "lib-force-field", inputs: ["size", "position"] }, { kind: "component", type: ArrowComponent, selector: "lib-arrow", inputs: ["direction", "pointToElement"] }, { kind: "directive", type: NgClass, selector: "[ngClass]", inputs: ["class", "ngClass"] }, { kind: "pipe", type: DirectionPipe, name: "direction" }, { kind: "component", type: TextOrTemplateComponent, selector: "lib-text-or-template", inputs: ["value"] }], animations: [
            trigger('fadeIn', [
                transition(':enter', [style({ opacity: 0 }), animate(500)]),
            ]),
        ] }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "17.0.8", ngImport: i0, type: EnjoyHintComponent, decorators: [{
            type: Component,
            args: [{ selector: 'lib-enjoyhint', standalone: true, imports: [
                        ForceFieldComponent,
                        ArrowComponent,
                        JsonPipe,
                        NgClass,
                        DirectionPipe,
                        TextOrTemplateComponent,
                    ], animations: [
                        trigger('fadeIn', [
                            transition(':enter', [style({ opacity: 0 }), animate(500)]),
                        ]),
                    ], providers: [provideWindow()], template: "@if (ffPositions(); as positions) {\n<lib-force-field\n  [size]=\"viewSize()\"\n  [position]=\"positions.top\"\n  [style.background-color]=\"options.backdropColor\"\n></lib-force-field>\n<lib-force-field\n  [size]=\"viewSize()\"\n  [position]=\"positions.left\"\n  [style.background-color]=\"options.backdropColor\"\n></lib-force-field>\n<lib-force-field\n  [size]=\"viewSize()\"\n  [position]=\"positions.right\"\n  [style.background-color]=\"options.backdropColor\"\n></lib-force-field>\n<lib-force-field\n  [size]=\"viewSize()\"\n  [position]=\"positions.bottom\"\n  [style.background-color]=\"options.backdropColor\"\n></lib-force-field>\n} @if (step()?.event === 'next') {\n<!-- If we just want them to click on the 'Next' button we want to disable events over the focused element -->\n<lib-force-field\n  [size]=\"elementBounds()\"\n  [position]=\"elementBounds()\"\n  [style.background-color]=\"'transparent'\"\n>\n</lib-force-field>\n}\n\n<ng-template #instructions>\n  <div\n    class=\"instructions\"\n    @fadeIn\n    [ngClass]=\"{\n    top: (position() | direction) === 'top',\n    left: (position() | direction) === 'left',\n    right: (position() | direction) === 'right',\n    bottom: (position() | direction) === 'bottom',\n  }\"\n  [style.maxWidth]=\"instructionsWidth()\"\n  >\n    <lib-text-or-template [value]=\"step()?.description\">\n      <ng-template let-description>\n        <label [style.fontFamily]=\"options.fontFamily\">\n          {{ description }}\n        </label>\n      </ng-template>\n    </lib-text-or-template>\n    @if (step()?.details) {\n    <lib-text-or-template [value]=\"step()?.details\">\n      <ng-template let-details>\n        <p [style.fontFamily]=\"options.fontFamily\">\n          {{ details }}\n        </p>\n      </ng-template>\n    </lib-text-or-template>\n    }\n    @if (focusElement(); as element) {\n      <lib-arrow [direction]=\"position() | direction\" [pointToElement]=\"focusElement()\"></lib-arrow>\n    }\n    <div class=\"buttons\">\n      @if (ref.tutorial.hasPrevious()) {\n      <button\n        class=\"previous\"\n        (click)=\"previous()\"\n        [style.fontFamily]=\"options.fontFamily\"\n        [ngClass]=\"step()?.previousButton?.className\"\n      >\n        {{ step()?.previousButton?.text ?? options.previousButton.text }}\n      </button>\n      } @if (step()?.event === 'next') {\n      <button\n        class=\"next\"\n        (click)=\"next()\"\n        [style.fontFamily]=\"options.fontFamily\"\n        [ngClass]=\"step()?.nextButton?.className\"\n      >\n        {{ step()?.nextButton?.text ?? options.nextButton.text }}\n      </button>\n      } @if (!step()?.hideSkip) {\n      <button\n        class=\"skip\"\n        (click)=\"skip()\"\n        [style.fontFamily]=\"options.fontFamily\"\n        [ngClass]=\"step()?.skipButton?.className\"\n      >\n        {{ step()?.skipButton?.text ?? options.skipButton.text }}\n      </button>\n      }\n    </div>\n  </div>\n</ng-template>\n", styles: ["label{color:#fff;font-size:2rem}p{color:#fff;font-size:1.5rem}.instructions{display:grid;grid-template-rows:repeat(4,auto);grid-template-columns:auto;gap:.5rem;justify-content:start;justify-items:start;padding:.75rem;max-width:25vw}.instructions label{grid-column:1;grid-row:1}.instructions p{grid-column:1;grid-row:2}.instructions lib-arrow{grid-column:1;grid-row:3}.instructions .buttons{grid-column:1;grid-row:4}.instructions.left{justify-content:end;justify-items:end}.instructions.right{justify-content:start;justify-items:start}.instructions.top{justify-content:center;justify-items:center}.instructions.top .buttons{grid-row:1}.instructions.top label{grid-row:2}.instructions.top p{grid-row:3}.instructions.top lib-arrow{grid-row:4}.instructions.bottom{justify-content:center;justify-items:center}.instructions.bottom lib-arrow{grid-row:1}.instructions.bottom label{grid-row:2}.instructions.bottom p{grid-row:3}.instructions.bottom .buttons{grid-row:4}.buttons{display:flex;flex-direction:row;gap:.5rem}.buttons button{background-color:transparent;border:1px solid white;border-radius:5px;color:#fff;font-size:.75rem;width:5rem;padding:.5rem;transition:all .25s ease-in-out;cursor:pointer}.buttons button.next{background-color:green;color:#fff}.buttons button:hover{background-color:#fff;color:#000}\n"] }]
        }], ctorParameters: () => [{ type: EnjoyHintRef }, { type: Window }, { type: i2.OverlayPositionBuilder }, { type: i2.Overlay }, { type: i0.ViewContainerRef }, { type: i0.DestroyRef }, { type: i0.NgZone }], propDecorators: { instructions: [{
                type: ViewChild,
                args: ['instructions']
            }], opacity: [{
                type: HostBinding,
                args: ['style.opacity']
            }] } });

class Tutorial {
    constructor(steps) {
        this._stepIndex = signal(0);
        this.stepIndex = this._stepIndex.asReadonly();
        this.step = computed(() => this.steps[this.stepIndex()]);
        this.hasPrevious = computed(() => this.stepIndex() > 0);
        const lastStep = steps.at(-1);
        if (lastStep) {
            lastStep.nextButton = {
                text: 'Finish',
                ...(lastStep.nextButton ?? {}),
            };
            if (lastStep.event === 'next') {
                lastStep.hideSkip = lastStep.hideSkip ?? true;
            }
        }
        this.steps = steps;
        this.startHook();
    }
    reset() {
        this._stepIndex.set(0);
    }
    setStepIndex(index) {
        const normalizedIndex = Math.max(0, Math.min(index, this.steps.length - 1));
        this._stepIndex.set(normalizedIndex);
    }
    nextStep() {
        this.endHook();
        this._stepIndex.update((index) => index + 1);
        this.startHook();
    }
    startHook() {
        const currentStepIndex = this._stepIndex();
        const currentStep = this.step();
        try {
            currentStep?.stepStart?.();
        }
        catch (e) {
            console.error(`Error executing stepEnd hook for step ${currentStepIndex}`, e, { currentStep });
        }
    }
    endHook() {
        const newStepIndex = this._stepIndex();
        const newStep = this.step();
        try {
            newStep?.stepEnd?.();
        }
        catch (e) {
            console.error(`Error executing stepStart hook for step ${newStepIndex}`, e, { newStep });
        }
    }
    previousStep() {
        this._stepIndex.update((index) => index - 1);
    }
}

class EnjoyHintService {
    /** @ignore */
    constructor(overlay, elements, originalOverflow) {
        this.overlay = overlay;
        this.elements = elements;
        this.originalOverflow = originalOverflow;
    }
    /**
     * Run an interactive tutorial
     * @param steps the tutorial steps to run
     * @param options optional object to override the default behavior
     * @returns a promise resolving when the tutorial is closed; resolves to `true` if the tutorial was completed, `false` if it was skipped
     */
    async runTutorial(steps, options) {
        const tutorial = new Tutorial(steps);
        const enjoyHintRef = new EnjoyHintRef(tutorial, options);
        const overlayRef = this.overlay.create({
            hasBackdrop: false,
            disposeOnNavigation: true,
            positionStrategy: this.overlay.position().global(),
            panelClass: 'ng-enjoyhint-overlay',
        });
        const zIndex = options?.overlayZIndex;
        if (zIndex !== undefined) {
            const globalOverlayWrapper = overlayRef.hostElement;
            const cdkOverlayContainer = globalOverlayWrapper.parentElement;
            [globalOverlayWrapper, cdkOverlayContainer].forEach((e) => e?.style.setProperty('z-index', zIndex.toString()));
        }
        const portal = new ComponentPortal(EnjoyHintComponent, null, Injector.create({
            providers: [{ provide: EnjoyHintRef, useValue: enjoyHintRef }],
        }));
        try {
            this.elements.body.style.overflow = 'hidden';
            overlayRef.attach(portal);
            return await firstValueFrom(enjoyHintRef.onClose);
        }
        finally {
            this.elements.body.style.overflow = this.originalOverflow;
            overlayRef.detach();
            overlayRef.dispose();
        }
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "17.0.8", ngImport: i0, type: EnjoyHintService, deps: "invalid", target: i0.ɵɵFactoryTarget.Injectable }); }
    static { this.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "17.0.8", ngImport: i0, type: EnjoyHintService, providedIn: 'root', useFactory: (overlay) => new EnjoyHintService(overlay, new Elements(), getComputedStyle(document.body).overflow), deps: [{ token: Overlay }] }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "17.0.8", ngImport: i0, type: EnjoyHintService, decorators: [{
            type: Injectable,
            args: [{
                    providedIn: 'root',
                    deps: [Overlay],
                    useFactory: (overlay) => new EnjoyHintService(overlay, new Elements(), getComputedStyle(document.body).overflow),
                }]
        }], ctorParameters: () => [{ type: i2.Overlay }, { type: Elements }, { type: undefined }] });

/*
 * Public API Surface of ng-enjoyhint-lib
 */

/**
 * Generated bundle index. Do not edit.
 */

export { EnjoyHintService };
//# sourceMappingURL=ng-enjoyhint.mjs.map
